# *****************************************************************
#
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2018, 2019. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# *****************************************************************

from __future__ import print_function
import os
import time
import argparse

## SPARK

from pyspark import SparkConf, SparkContext
from pyspark.sql.session import SparkSession

defaultPath = os.getcwd()
CLI=argparse.ArgumentParser()
CLI.add_argument(
    "--use_gpu",
    action = 'store_true',
    default=False
)
CLI.add_argument(
   "--data_path",
   type=str,
   default=defaultPath
)
CLI.add_argument(
   "--num_partitions",
   type=int,
   default= 2
)
CLI.add_argument(
   "--use_balanced_class_weights",
   action = 'store_true',
   default=False
)

args = CLI.parse_args()

if args.use_balanced_class_weights:
  class_weights = "balanced"
else:
  class_weights = None
# Set your own master (e.g 'spark://zhcc011.zurich.ibm.com:7077') or use 'local'
# Configure Spark (2 executors, local standalone mode)
# If one wants to run spark on a cluster, use your own spark configuration (set master , appName, etc...) 
conf = SparkConf()\
.set("spark.default.parallelism",args.num_partitions)\
.set("spark.cores.max",args.num_partitions)\
.setAppName("criteoKaggle2014-duality")\
.set("spark.executor.extraJavaOptions","-XX:MaxDirectMemorySize=8192m")\
.set("spark.driver.maxResultSize",8192)
# Create spark config
sc = SparkContext(conf=conf)
# Configure Spark
spark = SparkSession(sc)

# disable spark INFO logging - comment out to enable
log4j = sc._jvm.org.apache.log4j
log4j.LogManager.getRootLogger().setLevel(log4j.Level.ERROR)

# data paths
# Please use the absolute path of the file if you wish to run the example on a distributed mode
root_filename = args.data_path + '/data/criteo.kaggle2014'
train_filename_snap = root_filename+'-train_sparse.snap'
train_filename_snap_t = root_filename+'-train_sparse.snap.t'
test_filename_snap = root_filename+'-test_sparse.snap'

## snapML

from snap_ml_spark import DatasetReader
from snap_ml_spark import LogisticRegression as snapml_LogisticRegression
from snap_ml_spark.Metrics import accuracy

# Load training data
train_data_snap = DatasetReader().setFormat("snap").load(train_filename_snap)

# Load test data
test_data_snap = DatasetReader().setFormat("snap").load(test_filename_snap)


## DUAL LR

# Create snapML Logistic Regression
snapml_lr_dual = snapml_LogisticRegression(max_iter=500, regularizer=1.0, verbose=True, dual=True, use_gpu=args.use_gpu, class_weights=class_weights, return_training_history='summary')

# Fit the model and time it
snapml_t0 = time.time()
dual_training_loss = snapml_lr_dual.fit(train_data_snap)[0]['train_obj']
snapml_time_dual = time.time() - snapml_t0

# Perform inference on test data
pred = snapml_lr_dual.predict(test_data_snap)

# Compute accuracy
snapml_accuracy_dual = accuracy(pred)

# Print off SnapML result
print('dual LR snapML accuracy: %.4f' %snapml_accuracy_dual, ", time: %.2f" % snapml_time_dual, 's')

## PRIMAL LR

train_data_snap_t = DatasetReader().setFormat("snap").load(train_filename_snap_t)

snapml_lr_primal = snapml_LogisticRegression(max_iter=500, regularizer=1.0, verbose=True, dual=False, use_gpu=args.use_gpu, class_weights=class_weights, return_training_history='summary')

# Fit the model and time it
snapml_t0 = time.time()
primal_training_loss = snapml_lr_primal.fit(train_data_snap_t)[0]['train_obj']
snapml_time_primal = time.time() - snapml_t0

# Perform inference on test data
pred = snapml_lr_primal.predict(test_data_snap)

# Compute accuracy
snapml_accuracy_primal = accuracy(pred)

# Print off SnapML result
print('primal LR snapML accuracy: %.4f' %snapml_accuracy_primal, ", time: %.2f" % snapml_time_primal, 's')

# Print off duality gap
print('\nprimal LR snapML training loss: %.2f' % primal_training_loss, 'dual LR snapML training loss: %.2f' % dual_training_loss, 'duality gap: %.2f' % (primal_training_loss-(-dual_training_loss)))
