# *****************************************************************
#
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2018. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# *****************************************************************

from __future__ import print_function
import argparse
import os

defaultPath = os.getcwd()
CLI=argparse.ArgumentParser()
CLI.add_argument(
   "--data_path",
   type=str,
   default=defaultPath
)
args = CLI.parse_args()
# Load the data
print('loading HIGGS dataset...')
from sklearn.datasets import load_svmlight_file
filename = args.data_path + "/data/HIGGS"
X,y = load_svmlight_file(filename)

print('splitting data...')
# Make the train-test split
from sklearn.model_selection import train_test_split
X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.25, random_state=42)

# Convert to numpy ararys
print('converting to dense format...')
import numpy as np
X_train = np.array(X_train.todense())
X_test  = np.array(X_test.todense())

# Normalize the training data
print('normalizing data...')
from sklearn.preprocessing import normalize
X_train = normalize(X_train, axis=1, norm='l1')
X_test  = normalize(X_test,  axis=1, norm='l1')

# Save to csv files
# X_train_data = np.column_stack((y_train,X_train))
# X_test_data = np.column_stack((y_test,X_test))
# print 'Writing training data to csv file...'
# np.savetxt(filename + '-train.csv', X_train_data, delimiter=',')
# print 'Writing test data to csv file...'
# np.savetxt(filename + '-test.csv', X_test_data, delimiter=',')

# # Save to libsvm files (index must start from 1)
# from sklearn.datasets import dump_svmlight_file
# dump_svmlight_file(X_train,y_train, filename+'-train.libsvm', zero_based=False)
# dump_svmlight_file(X_test,y_test, filename+'-test.libsvm', zero_based=False)

# # Save to dense snap format
from snap_ml_spark.Utils import dump_to_snap_format
# from scipy import sparse
# Write to sparse snap format
# dump_to_snap_format(sparse.csr_matrix(X_train),y_train, filename+'-train_sparse.snap')
# dump_to_snap_format(sparse.csr_matrix(X_train),y_train, filename+'-train_sparse.snap.t', transpose=True)
# dump_to_snap_format(sparse.csr_matrix(X_test),y_test, filename+'-test_sparse.snap')
# Write to dense snap format
dump_to_snap_format(X_train,y_train, filename+'-train_dense.snap')
dump_to_snap_format(X_train,y_train, filename+'-train_dense.snap.t', transpose=True)
dump_to_snap_format(X_test,y_test, filename+'-test_dense.snap')

